package bdledger.api;

import bdledger.api.grpc.ledger.GetLedgersResponse;
import bdledger.api.grpc.common.TransactionType;
import io.grpc.Status;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import java.util.concurrent.ExecutionException;
import java.util.stream.IntStream;

import static org.junit.jupiter.api.Assertions.*;

@DisplayName("Transaction Ledger tests")
class ClientTests {

  private static final String ledger = "test";
  private String[] ledgers = new String[]{"first", "second", "third"};

  private static Client client;

  @BeforeAll
  static void initAll() {
    client = new Client("localhost", 10000);
  }

  @Test
  @DisplayName("ClientVersion#1")
  void clientVersion1() throws InterruptedException, ExecutionException {
    assertEquals(
        "TxLedgerGo/v0.0.1alpha/darwin/go1.11", client.clientVersion().get().getVersion());
  }

  @Test
  @DisplayName("CreateLedger#1")
  void createLedger1() throws InterruptedException, ExecutionException {
    assertTrue(client.createLedger(ledger).get().getOk());
  }

  @Test
  @DisplayName("CreateLedger#2")
  void createLedger2() {
    Throwable e = assertThrows(Exception.class, () -> client.createLedger("").get());
    Status s = Status.fromThrowable(e);
    assertEquals(Status.Code.INVALID_ARGUMENT, s.getCode());
    assertEquals("name must not be empty", s.getDescription());
  }

  @Test
  @DisplayName("GetLedgers#1")
  void getLedgers1() throws InterruptedException, ExecutionException {
    GetLedgersResponse r = client.getLedgers().get();
    assertEquals(3, r.getLedgersCount());
    String[] expected = new String[]{"first", "second", "third"};
    assertAll(
        IntStream.range(0, 3).boxed().map(i -> () -> assertEquals(ledgers[i], r.getLedgers(i))));
  }

  @Test
  @DisplayName("SendTransaction#1")
  void sendTransaction1() throws InterruptedException, ExecutionException {
    String hash =
        Utils.byteArrayToHexString(
            client
                .sendTransaction(
                    ledger,
                    TransactionType.MESSAGE,
                    "f00dcafef00dcafef00dcafef00dcafef00dcafe",
                    "feedbabefeedbabefeedbabefeedbabefeedbabe",
                    Utils.hexStringToByteArray("deadbeef"))
                .get()
                .getHash()
                .toByteArray());
    assertEquals("d15ea5edd15ea5edd15ea5edd15ea5edd15ea5ed", hash);
  }

  @Test
  @DisplayName("SendTransaction#2")
  void sendTransaction2() {
    Throwable e =
        assertThrows(
            Exception.class,
            () ->
                client
                    .sendTransaction(ledger, TransactionType.MESSAGE, null, "50bada55", null)
                    .get());
    Status s = Status.fromThrowable(e);
    assertEquals(Status.Code.INVALID_ARGUMENT, s.getCode());
    assertEquals("Multiple invalid arguments", s.getDescription());
  }

  @AfterAll
  static void teadDwonAll() throws InterruptedException {
    client.shutdown();
  }
}
